<?php namespace Engine\Job\Publish\Services\Posts;

use Exception;
use Engine\Job\Publish\Handlers\Couch\CouchHandler;
use Engine\Job\Publish\Services\Posts\Schemas\PostSchema;
use Engine\Job\Publish\Services\Posts\Schemas\DossierSchema;
use Engine\Job\Publish\Services\Posts\Schemas\CollectionSchema;
use Engine\Job\Publish\Support\Post as PostSupport;
use Illuminate\Http\Response;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Post\Collection\Collection;
use Stage\Post\Dossier\Dossier;
use Stage\Post\Post;
use Stage\Optimus;

class Migrate extends \Engine\Job\Publish\Services\Migrate {

    /**
     * @var Account
     */
    private $account;

    /**
     * @var CouchHandler
     */
    private $couchHandler;

    /**
     * @var Optimus
     */
    protected $optimus;

    public function __construct(Account $account, Config $config)
    {
        $this->account = $account;
        $this->config = $config;
    }

    public function setCouchHandler(CouchHandler $baseUrl)
    {
        $this->couchHandler = $baseUrl;
    }

    public function setOptimus(Optimus $optimus)
    {
        $this->optimus = $optimus;
    }

    public function start($fresh = false)
    {

        $couchHandler = $this->couchHandler;

        // Get all the changed Posts
        $changed = PostSupport::changed();
        // $changed = [['type' => 1, 'action_type' => 1, 'id' => 24, 'id_sync' => '', 'id_rev_sync' => '', 'update_date' => '', 'register_date' => '']];

        foreach ($changed as $ch)
        {

            // This id could be a post id, or dossier ir, or collection id
            $id = $ch['id'];
            $type = $ch['type'];
            $idSync = $ch['id_sync'];
            $idRev = $ch['id_rev_sync'];
            $actionType = $ch['action_type'];
            $registerDate = $ch['register_date'];
            $updateDate = $ch['update_date'];

            $data = [];

            if( $actionType != parent::ACTION_DELETE )
            {

                switch ($type) {
                    case parent::TYPE_COLLECTION:

                        /** @var Collection $collection */
                        $collection = Collection::find($id);

                        // Create a new DossierSchema
                        $collection = new CollectionSchema($this->account, $this->config, $collection, $registerDate, $updateDate);

                        // Build it
                        $data = $collection->build();

                        break;

                    case parent::TYPE_DOSSIER:

                        // Find the Dossier
                        /** @var Dossier $dossier */
                        $dossier = Dossier::find($id);

                        // Create a new DossierSchema
                        $dossier = new DossierSchema($this->account, $this->config, $dossier, $registerDate, $updateDate);

                        // Build it
                        $data = $dossier->build();

                        break;

                    case parent::TYPE_POST:

                        /** @var Post $post */
                        $post = Post::find($id);

                        // Create a new DossierSchema
                        $post = new PostSchema($this->account, $this->config, $post, $registerDate, $updateDate, $this->optimus);

                        // Build it
                        $data = $post->build();

                        break;

                }
            }

            switch ($actionType)
            {
                case parent::ACTION_CREATE:

                    if( $data )
                    {
                        try {
                            // Save
                            list($id_, $rev_) = $this->couchHandler->getCouchDBClient()->postDocument($data);

                            // Sync with mysql database
                            PostSupport::sync($id, $id_, $rev_, $type);
                        } catch (Exception $e){
                            throw $e;
                        }
                    }
                    break;

                case parent::ACTION_UPDATE:

                    // Check if the idSync is not empty
                    if( !empty($idSync) )
                    {
                        try {
                            // Update
                            list($id_, $rev_) = $this->couchHandler->getCouchDBClient()->putDocument($data, $idSync, $idRev);

                            // Sync with mysql database
                            PostSupport::sync($id, $id_, $rev_, $type);

                        } catch (Exception $e){
                            throw $e;
                        }
                    }

                    break;

                case parent::ACTION_DELETE:

                    try {

                        $doc = $this->couchHandler->getCouchDBClient()->findRevisions($idSync);
                        if( $doc->status == Response::HTTP_OK )
                        {
                            $body = $doc->body;
                            foreach ($body as $b)
                            {
                                $rev_ = $b['ok']['_rev'];
                                $this->couchHandler->getCouchDBClient()->deleteDocument($idSync, $rev_);

                            }

                        }

                    } catch (Exception $e){
                        throw $e;
                    }

                    break;

            }
        }

        // Create design documents if it the database is fresh
        if( $fresh )
        {
            $this->createDesignDocuments($couchHandler);
        }

        // Compact database
        // $couchHandler->getCouchDBClient()->compactDatabase();

    }

    /**
     * @param CouchHandler $couchHandler
     */
    public function createDesignDocuments(CouchHandler $couchHandler)
    {
        // _design/$designName
        $client = $couchHandler->getCouchDBClient();
        $designName = 'collections';

        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\Collections());

        $designName = 'collections_dossiers';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiers());

//        $designName = 'collections_dossier';
//        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossier());

        $designName = 'collections_dossiers_posts';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiersPosts());

        $designName = 'collections_dossiers_post';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiersPost());

        $designName = 'collections_dossiers_post_by_slug';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiersPostBySlug());

        $designName = 'collections_latest';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionLatestPosts());

        $designName = 'all_docs';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AllDocs());
    }

}
