<?php namespace Engine\Job\Publish\Services\Posts\Schemas;

use DateTime;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use stdClass;
use Stage\Post\Collection\Collection;
use Stage\Post\Dossier\Dossier;
use Stage\Post\Post;

class CollectionSchema {

    /**
     * @var Collection
     */
    public $collection;

    const DATETIME_FORMAT = 'Y/m/d H:i:s O';

    public function __construct(Account $account, Config $config, Collection $collection, $registerDate, $updateDate)
    {
        $this->account = $account;
        $this->collection = $collection;
        $this->config = $config;
        $this->baseUrl = $config->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $this->account->getToken();
        $this->registerDate = (new DateTime($registerDate))->format(self::DATETIME_FORMAT);
        $this->updateDate = (new DateTime($updateDate))->format(self::DATETIME_FORMAT);
    }

    public function build()
    {
        $allIsoLanguages = $this->allIsoLanguages();

        $data = [
            'id' => (int) $this->collection->getId(),
            'name' => $this->collection->getName(),
            'domain' => $this->collection->getDomain(),
            'dossiers' => ['count' => $this->dossiersCount()],
            'tags' => $this->tags($allIsoLanguages),
            'authors' => $this->authors($allIsoLanguages),
        ];

        if( $this->collection->getDefaultDossier() instanceof Dossier )
        {

            /** @var Dossier $dossier */
            $dossier = $this->collection->getDefaultDossier();
            $data['dossier'] = [    // Its the default dossier of the collection
                'id' => $dossier->getId(),
                'name' => $dossier->getName(),
                'random' => $dossier->getRandom(),
                'slug' => str_slug(html_entity_decode($dossier->getManagementName(), ENT_COMPAT, "UTF-8"), "-")
            ];

        }

        return ["id" => $this->collection->getId(),
                 "data" => $data,
                 "created_at" => $this->registerDate,
                 "updated_at" => $this->updateDate,
                 "type" => "collection"];
    }

    public function allIsoLanguages()
    {
        // Get all iso languages of all Posts
        return Post::getAllIsoLanguages();
    }

    public function tags($isoLanguages)
    {
        $tags = [];
        foreach ($isoLanguages as $isoLanguage)
        {
            $iso = $isoLanguage['iso_language'];

            $tagsList = \Engine\Job\Publish\Support\Post::postsTags($this->collection->getId(), $iso);
            foreach ($tagsList as $t)
            {

                $found = false;
                $tag = [
                    'id' => (int) $t['id_tag'],
                    'name' => $t['name'],
                    'slug' => $t['name_stripped'],
                    'count' => (int) $t['total'] // Total number of Posts with this Tag
                ];

                for ( $i = 0 ; $i <= sizeof($tags) - 1 ; $i++ ) {
                    if( $tags[$i]['iso'] == $iso) {
                        $tags[$i]['list'][] = $tag;
                        $found = true;
                    }
                }

                if( !$found ) {
                    array_push($tags, ['iso' => $iso, 'list' => [$tag]]);
                }
            }
        }

        return $tags;
    }

    public function dossiersCount()
    {
        $query = new \Stage\Post\Collection\Query();
        $query->term = "";
        $query->from = 0;
        $query->to = -1;

        // Get the collection dossiers
        return sizeof($this->collection->dossiers($query));

    }

    public function authors($isoLanguages)
    {
        $authors = [];
        foreach ($isoLanguages as $isoLanguage)
        {
            $iso = $isoLanguage['iso_language'];

            $authorsList = \Engine\Job\Publish\Support\Post::postsAuthors($this->collection->getId(), $iso);

            foreach ($authorsList as $a)
            {

                $found = false;

                $author = new stdClass();
                $author->id = (int)$a['id_author'];
                $author->name = $a['name'];
                $author->role = $a['role'];
                $author->biography = $a['biography'];
                $author->email = $a['email'];

                $posts = new stdClass();
                $posts->written = (int)$a['total']; // Quantos Posts escreveu este autor dentro de uma collection X e dentro de cada Iso Y
                $author->posts = $posts;

                $img = new stdClass();
                $img->file = $a['photo'];
                $img->baseUrl = $this->getBaseUrl();
                $author->photo = $img;

                $s = new stdClass();
                $s->facebook = $a['facebook'];
                $s->twitter = $a['twitter'];
                $s->linkedin = $a['linkedin'];
                $s->instagram = $a['instagram'];
                $s->skype = $a['skype'];
                $s->website = $a['website'];

                $author->social = $s;

                for ( $i = 0 ; $i <= sizeof($authors) - 1 ; $i++ ) {
                    if( $authors[$i]['iso'] == $iso) {
                        $authors[$i]['list'][] = $author;
                        $found = true;
                    }
                }

                if( !$found ) {
                    array_push($authors, ['iso' => $iso, 'list' => [$author]]);
                }

            }

        }

        return $authors;
    }

    public function getBaseUrl()
    {
        return $this->baseUrl;
    }
}