<?php namespace Engine\Job\Publish\Services\Posts\Schemas;

use DateTime;
use Engine\Engine;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Helper;
use Stage\Post\Dossier\Dossier;
use Stage\Asset\Query;
use Stage\Post\L10n\L10n;
use Stage\Post\Post;
use stdClass;
use Stage\Optimus;
use Stage\Video\Video;

class PostSchema {

    /**
     * @var Post
     */
    private $post;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Account
     */
    private $account;

    /**
     * @var
     */
    private $baseUrl;

    /**
     * @var Optimus
     */
    protected $optimus;

    const DATETIME_FORMAT = 'Y/m/d H:i:s O';

    /**
     * @var string
     */
    private $playHostUrl;

    public function __construct(Account $account, Config $config, Post $post, $registerDate, $updateDate, $optimus)
    {
        $this->account = $account;
        $this->config = $config;
        $this->post = $post;
        $this->baseUrl = $config->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $this->account->getToken();
        $this->videoBaseUrl = $config->get(Config::VIDEO_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();
        $this->assetBaseUrl = $config->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();
        $this->registerDate = (new DateTime($registerDate))->format(self::DATETIME_FORMAT);
        $this->updateDate = (new DateTime($updateDate))->format(self::DATETIME_FORMAT);
        $this->optimus = $optimus;
        $this->playHostUrl = $config->get(Config::PLAY_HOST);
    }

    private function getBaseUrl()
    {
        return $this->baseUrl;
    }

    private function getVideoBaseUrl()
    {
        return $this->videoBaseUrl;
    }

    private function getAssetBaseUrl()
    {
        return $this->assetBaseUrl;
    }

    public function build()
    {

        // Get all post iso languages
        //$isos = $this->post->getIsoLanguages();

        // Get the dossier where the Post exist
        /** @var Dossier[] $dossiers */
        // $dossiers = $this->post->dossiers();
        $dossiers = [];
        $pairs = \Engine\Job\Publish\Support\Post::existIn($this->post->getId());
        foreach ($pairs as $pair)
        {
            $dossiers[] = Dossier::find($pair['id_dossier']);
        }

        $l10n = $this->l10ns();
        $isos = array_map(function($l10n){
            return $l10n->iso;
        }, $l10n);

        $data = ['id' => $this->post->getId(),
            'data' => [
                'id' => (int) $this->post->getId(),
                'l10n' => $l10n,   // Filtrar so os L10n's que estao no estado 5
            ],
            'dossiers' => $this->dossiers($dossiers),
            'iso' => $isos,
            'created_at' => $this->registerDate,
            'updated_at' => $this->updateDate,
            'on' => $this->on(),
            'type' => "post"];

        // If there is no L10n's then return null and not insert the Post
        return sizeof($l10n) ? $data : null;
    }

    public function allIsoLanguages()
    {
        // Get all iso languages of all Posts
        return Post::getAllIsoLanguages();
    }

    public function on()
    {
        $pairs = \Engine\Job\Publish\Support\Post::existIn($this->post->getId());
        return array_map(function($pair){
            return [(int)$pair['id_collection'], (int)$pair['id_dossier']];
        }, $pairs);
    }

    private function dossiers($dossiers)
    {
        $data = [];
        /** @var Dossier $d */
        foreach ($dossiers as $d)
        {

            $dossier = new stdClass();
            $dossier->id = $d->getId();
            $dossier->name = html_entity_decode($d->getName(), ENT_COMPAT, "UTF-8");
            $dossier->random = $d->getRandom();
            $dossier->l10n = array_map(function($l10n){ return $l10n->toArray(); }, $d->getL10n());
            $dossier->slug = str_slug(html_entity_decode($d->getName(), ENT_COMPAT, "UTF-8"), "-");

            $isoLanguages = $this->allIsoLanguages();
            // $isoLanguages = $this->isoLanguages;

            $temp = [];
            foreach ($isoLanguages as $iso)
            {
                $key = $iso['iso_language'];
                $tags = \Engine\Job\Publish\Support\Post::postsDossiersTags($d->getId(), $key);
                foreach ($tags as $tag)
                {
                    $temp[] = [
                        'id' => (int)$tag['id_tag'],
                        'name' => $tag['name'],
                        'slug' => $tag['name_stripped'],
                        'count' => (int) $tag['total']  // Count is the total number of times which the tag is used
                    ];
                }
            }

            $dossier->tags = $temp;

            $data[] = $dossier;
        }

        return $data;
    }

    private function l10ns()
    {
        $l10ns = $this->post->getL10n();

        $temp = [];
        /** @var L10n $l10n */
        foreach ($l10ns as $l10n)
        {
            if( $l10n->getState() == 5 ) {

                $l              = new stdClass();

                // Title
                $l->title       = $this->title($l10n->getTitle());

                // Description
                $l->description = $this->description($l10n->getDescription());

                // Slug
                $l->slug        = $this->slug($l10n->getTitle());

                // Iso
                $l->iso         = $this->iso($l10n->getIsoLanguage());

                // Image
                $l->image = $this->image($l10n->getImage());

                // Read time
                $l->readTime = $this->readTime($l10n->getBody());

                // Date
                $l->date = $this->datee($l10n->getRegisterDate(), $l10n->getPublishDate(), $l10n->getBeginDate());

                // Content
                $l->content = $this->bodies($l10n->getBody());

                // Tags
                $l->tags = $this->tags($l10n->getTags());

                // Metadata
                $l->metadata = $this->metadata($l10n->getMetadata());

                // Authors
                $l->authors = $this->authors($l10n->getAuthors());

                // Media
                $l->media = $this->media($l10n->getAssets(), $l10n->getAlbums(), $l10n->getVideos());

                $temp[] = $l;
            }
        }

        return $temp;
    }

    public function title($title)
    {
        return html_entity_decode($title, ENT_COMPAT, "UTF-8");
    }

    public function description($description)
    {
        return html_entity_decode($description, ENT_COMPAT, "UTF-8");
    }

    public function slug($title)
    {
        return str_slug(html_entity_decode($title, ENT_COMPAT, "UTF-8"), "-");
    }

    public function iso($iso)
    {
        return $iso;
    }

    public function image($image)
    {
        $i = new stdClass();
        $i->file = $image;
        $i->baseUrl = $this->getBaseUrl();

        return $i;
    }

    public function imageVideo($compose, $isLive, $image)
    {
        if( !$isLive )
        {
            $video = Video::find($compose->getIdVideo());
//            $this->sneakPeeks = $this->video->sneakPeaks();
        }

        $i = new stdClass();

        // If the Thumbnail is from Video
        if( !$isLive && $compose->getThumbnailChoice() == 1 )
        {
            $i->file = $video->getDefaultThumbnail() ? $video->getDefaultThumbnail() : "";
            $i->baseUrl = $this->getVideoBaseUrl();
        }
        else
        {
            $i->file = $image;
            $i->baseUrl = $this->getAssetBaseUrl();
        }

        return $i;
    }

    public function metadataVideo(/*Metadata*/ $metadata)
    {
        return [
            'image' => [
                'baseUrl' => $this->getAssetBaseUrl(),
                'file' => $metadata->getImage(),
                'alt' => $metadata->getImageAlt(),
                'title' => $metadata->getImageTitle(),
            ],
            'title' => $metadata->getTitle(),
            'description' => $metadata->getDescription(),
            'keywords' => $metadata->getKeywords()
        ];
    }

    public function readTime($bodies)
    {
        $readTime = 0;
        foreach($bodies as $body)
        {
            $text = strip_tags($body->getBody());
            $readTime += Helper::readTime($text);
        }

        return $readTime;
    }

    public function datee($registerDate, $publishDate, $beginDate)
    {
        $date = new stdClass();
        $date->register = (new DateTime($registerDate))->format(self::DATETIME_FORMAT);
        $date->publish = (new DateTime($publishDate))->format(self::DATETIME_FORMAT);
        $date->begin = (new DateTime($beginDate))->format(self::DATETIME_FORMAT);

        return $date;
    }

    public function tags($tags)
    {

        // Filter to only public tags
        $arrayTags = array_map(function ($tag){
            return $tag;
        }, array_values(array_filter($tags, function ($tag) {
            return $tag->getPublic() == true;
        })));

        $temp = [];
        foreach ($arrayTags as $tag)
        {
            $t = new stdClass();
            $t->id = $tag->getId();
            $t->name = $tag->getName();
            $t->slug = $tag->getNameStripped();
            $t->total = $tag->getTotal();
            $temp[] = $t;
        }

        return $temp;
    }

    public function bodies($bodies)
    {
        $b = [];
        foreach ($bodies as $body)
        {
            $b[] = $body->getBody();
        }

        return $b;
    }

    public function metadata(\Stage\Post\Metadata\Metadata $metadata)
    {
        $meta = new stdClass();
        $meta->title = $metadata->getTitle();
        $meta->description = $metadata->getDescription();
        $meta->keywords = str_replace(' ', '', $metadata->getKeywords());
        $meta->playmark = (bool) $metadata->getPlayMark();
        $meta->slug = $metadata->getUrl();
        //$meta->hash = $metadata->getHash();

        $img = new stdClass();
        $img->file = $metadata->getImage();
        $img->baseUrl = $this->getBaseUrl();
        $meta->image = $img;

        return $meta;
    }

    public function authors($authors)
    {
        $temp = [];
        foreach ($authors as $author)
        {
            $a = new stdClass();
            $a->id = $author->getId();
            $a->name = $author->getName();
            $a->email = $author->getEmail();

            $img = new stdClass();
            $img->file = $author->getPhoto();
            $img->baseUrl = $this->getBaseUrl();
            $a->image = $img;

            $a->l10n = [];
            foreach ($author->getL10n() as $l10n)
            {
                $r = new stdClass();
                $r->role = $l10n->getRole();
                $r->iso = $l10n->getIsoLanguage();
                $r->biography = $l10n->getBiography();
                $a->l10n[] = $r;
            }

            $s = new stdClass();
            $s->facebook = $author->getFacebook();
            $s->twitter = $author->getTwitter();
            $s->linkedin = $author->getLinkedin();
            $s->instagram = $author->getInstagram();
            $s->skype = $author->getSkype();
            $s->website = $author->getWebsite();

            $a->social = $s;
            $temp[] = $a;
        }

        return $temp;
    }

    public function media($assets, $albums, $videos)
    {

        $media = new stdClass();
        $media->images = [];
        $media->files = [];
        foreach ($assets as $asset)
        {
            $idStorage = $asset->getIdStorage();
            $idCdn = $asset->getIdCdn();

            // Convert idStorage to actual string path
            $assetsPath = \Stage\Account\Map::getStorage($idStorage, \Stage\Account\Map::ASSETS)->getStoragePath();
            $baseUrl = \Stage\Account\Map::getCdn($idCdn, \Stage\Account\Map::ASSETS)->getCdnPath();

            /** @var [] $paths */
            $paths = Engine::paths($this->account->getToken(), $assetsPath, $asset->getRegisterDate());

            $filePath = Engine::existIn($paths, $asset->getFilename());

            // $filePath = $this->config->get(Config::STORAGE_ASSETS) . DIRECTORY_SEPARATOR . $this->account->getToken() . DIRECTORY_SEPARATOR . $asset->getFilename();

            if( file_exists($filePath) )
            {
                $image = getimagesize($filePath) ? true : false;

                $file = new stdClass();
                $file->file = $asset->getFilename();
                $file->baseUrl = $this->getBaseUrl();
                $file->name = $asset->getName();
                $file->description = $asset->getDescription();
                $file->link = $asset->getLink();

                if( $image )
                {
                    $file->with = $asset->getWidth();
                    $file->height = $asset->getheight();
                    $media->images[] = $file;
                }
                else
                {
                    $media->files[] = $file;
                }
            }
        }

        $media->albums = [];
        foreach ($albums as $album)
        {
            $a = new stdClass();
            $a->id = $album->getId();
            $a->name = $album->getName();
            $a->description = $album->getDescription();

            $cover = new stdClass();
            $cover->baseUrl = $this->getBaseUrl();
            $cover->photo = $album->getCoverImage();
            if( !empty($cover->photo) )
                $a->cover = $cover;

            $count = new stdClass();
            $count->count = $album->getTotalImages();
            $a->assets = $count;
            $a->random = $album->getRandom();
            $a->link = $album->getLink();

            $query  = new Query();
            $query->term = "";
            $query->by = Query::ALL;
            $query->from = 0;
            $query->to = -1;
            $assets = $album->assets($query);

            $basePath = $this->config->get(Config::STORAGE_ASSETS) . DIRECTORY_SEPARATOR . $this->account->getToken();
            $a->assets = array_map(function($asset) use ($basePath) {

                $filePath = $basePath . DIRECTORY_SEPARATOR . $asset->getFilename();
                $temp = $asset->toArray();
                unset($temp['state']);

                if( file_exists($filePath) ) {
                    $image = getimagesize($filePath) ? true : false;

                    if (!$image) {
                        unset($temp['imageTitle']);
                        unset($temp['imageAlterText']);
                        unset($temp['width']);
                        unset($temp['height']);
                    }
                }
                return $temp;

            }, $assets);

            $media->albums[] = $a;
        }

        // Videos
        /** @var \Stage\Video\Compose\Compose $compose */
        foreach ($videos as $compose) {

            $composeAsArray = $compose->toArray();
            $composeAsArray['embedLink'] = $this->buildPlayHostUrl($this->playHostUrl, $this->account->getToken(), $compose->getId());
//            if( isset($composeAsArray['video']) )
//                unset($composeAsArray['video']);

            $isLive = $compose->getType() == 1;

            $packed = \Stage\Helper\Video\Hash::pack([0, 0, $compose->getId()]);
            $encoded = $this->optimus->encode($packed);
            $videoData = [
                'id' => $compose->getId(),
                'duration' => $isLive ? null : $composeAsArray['video']['duration'],
                'autoplay' => $compose->getAutoplay(),
                'hasInsideSocial' => $compose->hasInsideSocial(),
                'playInline' => $compose->playInline(),
                'playMark' => $compose->playMark(),
                'date' => [
                    'begin' => (new DateTime($compose->getBeginDate()))->format(self::DATETIME_FORMAT),
                    'end' => (new DateTime($compose->getEndDate()))->format(self::DATETIME_FORMAT),
                    'register' => (new DateTime($compose->getRegisterDate()))->format(self::DATETIME_FORMAT),
                    'publish' => (new DateTime($compose->getPublishDate()))->format(self::DATETIME_FORMAT),
                ],
                'publicId' => $encoded,
                'embedLink' => $this->buildPlayHostUrl($this->playHostUrl, $this->account->getToken(), $compose->getId())
            ];

            $profile = call_user_func_array([$compose->getType() == 1 ? '\Stage\Live\Profile\Profile' : '\Stage\Video\Profile\Profile','find'], [$compose->getIdProfile()]);

            $temp = [];
            /** @var L10n $l10n */
            foreach ($profile->getL10n() as $l10n)
            {
                $temp[] = [
                    //'id' => $l10n->getId(),
                    'isoLanguage' => $l10n->getIsoLanguage(),
                    'title' => $l10n->getTitle(),
                    'slug' => str_slug(html_entity_decode($l10n->getTitle(), ENT_COMPAT, "UTF-8"), "-"),
                    'description' => $l10n->getDescription(),
                    'image' => $this->imageVideo($compose, $isLive, $l10n->getImage()),
                    'metadata' => $this->metadataVideo($l10n->getMetadata()),
//                    'tags' => $this->tags($l10n),
//                    'body' => $this->body($l10n),
//                    'assets' => $this->assets($l10n),
//                    'albums' => $this->albums($l10n),
                ];
            }

            $videoData['profile'] = ['l10n' => $temp];

            // Videos
            $media->videos[] = (object) $videoData;
        }

        return $media;
    }

    public function buildPlayHostUrl($playHostUrl, $accountToken, $videoId) {
        $packed = \Stage\Helper\Video\Hash::pack([0, 0, $videoId]);

        // Hash the triple [idChannel, idPlaylist, idComposed]
        $encoded = $this->optimus->encode($packed);
        return $this->embedLink($playHostUrl, $accountToken, $encoded);
    }

    /**
     * @param $hash
     * @return string
     */
    private function embedLink($host, $accountToken, $hash)
    {
        return implode(DIRECTORY_SEPARATOR, [$host, $accountToken, $hash]);
    }
}