<?php namespace Engine\Job\Publish\Services\Videos\Schemas;

use DateTime;
use Engine\Job\Publish\Schema\Schema;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Video\Channel\Channel;
use Stage\Live\Channel\Channel as LiveChannel;
use Stage\Video\Compose\Compose;
use Stage\Video\Profile\L10n\Body;
use Stage\Video\Profile\L10n\L10n;
use Stage\Video\Profile\Metadata\Metadata;
use Stage\Video\Profile\Profile;
use Stage\Live\Profile\Profile as LiveProfile;
use Stage\Video\Sticker\Sticker;
use Stage\Video\Tag\Tag;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Video\Playlist\Playlist;
use Stage\Video\Sticker\Query as QuerySticker;
use Stage\Video\Video;
use Stage\Asset\Query;
use stdClass;

class ComposeSchema extends Schema {

    /**
     * @var Compose
     */
    private $compose;

    /**
     * @var Video
     */
    private $video;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Account
     */
    private $account;

    /**
     * @var
     */
    private $settings;

    /**
     * @var Account
     */
    private $isLive = false;

    public function __construct(Account $account, Config $config, $compose, $registerDate, $updateDate)
    {
        $this->isLive = $compose instanceof \Stage\Live\Compose\Compose ? true : false;
        $this->account = $account;
        $this->config = $config;
        $this->compose = $compose;
        $this->beginDate = (new DateTime($this->compose->getBeginDate()))->format(parent::DATETIME_FORMAT);
        $this->publishDate = (new DateTime($this->compose->getPublishDate()))->format(parent::DATETIME_FORMAT);
        $this->endDate = !$this->compose->getEndDate() ? null : (new DateTime($this->compose->getEndDate()))->format(parent::DATETIME_FORMAT);
        $this->registerDate = (new DateTime($registerDate))->format(parent::DATETIME_FORMAT);
        $this->updateDate =  !$updateDate ? null : (new DateTime($updateDate))->format(parent::DATETIME_FORMAT);

        $this->videoBaseUrl = $config->get(Config::VIDEO_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();
        $this->assetBaseUrl = $config->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();

        if( !$this->isLive )
        {
            $this->video = Video::find($this->compose->getIdVideo());
            $this->sneakPeeks = $this->video->sneakPeaks();
        }
    }

    private function getVideoBaseUrl()
    {
        return $this->videoBaseUrl;
    }

    private function getAssetBaseUrl()
    {
        return $this->assetBaseUrl;
    }

    public function build()
    {
        $profile = call_user_func_array([$this->isLive ? '\Stage\Live\Profile\Profile' : '\Stage\Video\Profile\Profile','find'], [$this->compose->getIdProfile()]);

        $temp = [];
        /** @var L10n $l10n */
        foreach ($profile->getL10n() as $l10n)
        {
            $temp[] = [
                //'id' => $l10n->getId(),
                'isoLanguage' => $l10n->getIsoLanguage(),
                'title' => $l10n->getTitle(),
                'slug' => str_slug(html_entity_decode($l10n->getTitle(), ENT_COMPAT, "UTF-8"), "-"),
                'description' => $l10n->getDescription(),
                'image' => $this->image($l10n->getImage()),
                'metadata' => $this->metadata($l10n->getMetadata()),
                'tags' => $this->tags($l10n),
                'body' => $this->body($l10n),
                'assets' => $this->assets($l10n),
                'albums' => $this->albums($l10n),
            ];
        }

        if( !$this->isLive )
        {
            $video = $this->composeVideo();
        }

        $data = [];
        $data['data'] = [
            'id' => $this->compose->getId(),
            //'name' => $video['name'],
            'duration' => $this->isLive ? null : $video['duration'],
            'type' => $this->compose->getType(),    // 0 - VOD, 1 - LIVE
            'autoplay' => $this->compose->getAutoplay(),
            'hasInsideSocial' => $this->compose->hasInsideSocial(),
            'playInline' => $this->compose->playInline(),
            'playMark' => $this->compose->playMark(),
            'profile' => ['l10n' => $temp],
            'playlists' => $this->playlists($this->compose),
            'labels' => $this->labels($this->compose),
            'stickers' => $this->compose->getOnSticker() ? $this->stickers($this->compose) : [],
            'date' => [
                'begin' => $this->beginDate,
                'end' => $this->endDate,
                'register' => $this->registerDate,
                'publish' => $this->publishDate,
            ],
            //'embedLink' => $this->embedLink($encoded)
        ];

        $data['id'] = $this->compose->getId();
        $data['on'] = $this->on();

        $data['channel_tags'] = [];
        foreach ($data['on'] as $channel)
        {
            $idChannel = $channel[0];
            $channel = !$this->isLive ? Channel::find($idChannel) : LiveChannel::find($idChannel);
            $data['channel_tags'][] = array_map(function($tag) use ($idChannel) {

                $tag = $tag->toArray();
                $t = [];
                $t['id'] = $tag['id'];
                $t['in_channel'] = $idChannel;
                $t['name'] = $tag['name'];
                $t['slug'] = $tag['nameStripped'];
                $t['count'] = $tag['total'];

                return $t;
            }, $channel->tags());
        }
        $data['channel_tags'] = array_values($data['channel_tags']);

        $types = [
            ['type' => 'application/dash+xml', 'extension' => '.mpd', 'name' => 'manifest'],        // DASH
            ['type' => 'application/x-mpegURL', 'extension' => '.m3u8', 'name' => 'manifest'],      // HLS
            ['type' => 'application/vnd.ms-sstr+xml', 'extension' => '', 'name' => 'Manifest'],     // SmoothStreaming
        ];

        $versionType = ['w','m'];

        // Is Live
        if( $this->isLive )
        {
            $data['stream'] = $this->stream($this->compose);

            // $videoName = pathinfo($this->video->getFilename())['filename'];
            $videoName = $data['stream']['stream'];
            $appName = $data['stream']['application'];
            $cdn = $data['stream']['cdn'] . DIRECTORY_SEPARATOR . $appName;

            foreach ($types as $type)
            {
                $src = implode(DIRECTORY_SEPARATOR, [rtrim($cdn, DIRECTORY_SEPARATOR), "smil:" . $videoName .".smil", $type['name'].$type['extension']]);
                $data['sources'][] = [
                    'src' => $src,
                    'type' => $type['type']
                ];
            }

        }
        else
        {
            $data['video'] = $this->video->toArray();
            $data['video']['date']['register'] = (new DateTime($data['video']['registerDate']))->format(parent::DATETIME_FORMAT);
            unset($data['video']['registerDate']);
            $data['video']['versions'] = array_collapse($this->versions($this->video));

            $videoName = pathinfo($this->video->getFilename())['filename'];

            $this->videoCdn = $this->config->get(Config::VIDEO_CDN_ABR) . DIRECTORY_SEPARATOR . $this->config->get(Config::VIDEO_ABR_APPLICATION);
            if( isset($this->settings) && isset($this->settings[0]) && isset($this->settings[0]['application']) )
            {
                $this->videoCdn = $this->settings[0]['path_cdn_abr'] . DIRECTORY_SEPARATOR . $this->settings[0]['application'];
            }

            $cdn = $this->videoCdn;

            foreach ($versionType as $t)
            {
                foreach ($types as $type)
                {
                    $data['sources'][$t][] = [
                        'src' => implode(DIRECTORY_SEPARATOR, [rtrim($cdn, DIRECTORY_SEPARATOR), "smil:" . $videoName ."-$t.smil", $type['name'].$type['extension']]),
                        'type' => $type['type']
                    ];
                }
            }

            if( sizeof($this->sneakPeeks) )
            {
                $data['video']['sneakPeeks'] = array_map(function($sp){
                    $t = $sp->toArray();
                    $t['date']['register'] = (new DateTime($t['registerDate']))->format(parent::DATETIME_FORMAT);

                    unset($t['id']);
                    unset($t['registerDate']);

                    return $t;
                }, $this->sneakPeeks);
            }
        }

        $data['publish_at'] = $this->publishDate;
        $data['begin_at'] = $this->beginDate;
        $data['end_at'] = $this->endDate;
        $data['created_at'] = $this->registerDate;
        $data['updated_at'] = $this->updateDate;
        $data['is_live'] = (bool) $this->isLive;
        $data['type'] =  "composed";

        return $data;
    }

    public function stream(/*Compose*/ $compose)
    {
        if( isset($compose->toArray()['stream']) )
        {
            $data = $compose->toArray()['stream'];
            $data['src'] = implode(DIRECTORY_SEPARATOR, [$data['server'], $data['application'], $data['name'], $data['stream']]);
            unset($data['id']);
            return $data;
        }
        return [];
    }

    public function dash($cdn, $app, $stream)
    {
        return implode(DIRECTORY_SEPARATOR, [$cdn, $app, $stream, "manifest.mpd"]);
    }

    public function hls($cdn, $app, $stream)
    {
        return implode(DIRECTORY_SEPARATOR, [$cdn, $app, $stream, "manifest.m3u8"]);
    }

    public function smoothstreaming($cdn, $app, $stream)
    {
        return implode(DIRECTORY_SEPARATOR, [$cdn, $app, $stream, "manifest.m3u8"]);
    }

    public function image($image)
    {
        $i = new stdClass();

        // If the Thumbnail is from Video
        if( !$this->isLive && $this->compose->getThumbnailChoice() == 1 )
        {
            $i->file =  $this->video->getDefaultThumbnail() ? $this->video->getDefaultThumbnail() : "";
            $i->baseUrl = $this->getVideoBaseUrl();
        }
        else
        {
            $i->file = $image;
            $i->baseUrl = $this->getAssetBaseUrl();
        }

        return $i;
    }

    public function l10n(Profile $profile)
    {
        $l10ns = $profile->getL10n();
        return array_map(function ($l10n){
            $temp = $l10n->toArray();
            unset($temp['id']);
            // $temp['created_at'] = (new DateTime($temp['registerDate']))->format(parent::DATETIME_FORMAT);
            return $temp;
        }, $l10ns);
    }

    public function versions(Video $video)
    {


        $cdn            = $this->config->get(Config::VIDEO_CDN_VERSIONS);
        $source         = $this->config->get(Config::STORAGE_VIDEO);
        $versionsPath   = $this->config->get(Config::VIDEO_VERSIONS_PATH);
        $accountToken   = $this->account->getToken();

        $ver = [];

        // Publish to couch all the versions
        $versions = [\Stage\Video\Version::WEB, \Stage\Video\Version::MOBILE, \Stage\Video\Version::LOW_MOBILE];
        foreach ($versions as $v)
        {

            $ver[] = array_map( function($version) use ($video, $cdn, $source, $accountToken, $versionsPath) {

                // Its the filename without extension
                $fileName = pathinfo($version->getFilename())['filename'];
                $versionType = $version->getVersionType();
                $versionNumber = $version->getVersion();
                $versionExtension = $version->getExtension();

                $this->settings = \Stage\Account\Map::getVideoAbrSettings($this->account->getId(), $version->getIdCdn(), $version->getIdStorage());
                if( isset($this->settings[0]) && isset($this->settings[0]['cdn_versions']) && $this->settings[0]['cdn_versions'] == $version->getIdCdn() )
                {
                    $cdn = $this->settings[0]['path_cdn_versions'];
                }

                // Its the video url to the file
                $source = implode(DIRECTORY_SEPARATOR, [$cdn, $accountToken, $versionNumber, $versionType, $fileName]);

                // Its the video source on disk
                $file = implode(DIRECTORY_SEPARATOR, [$versionsPath, $accountToken, $versionNumber, $versionType, $fileName]);

                // Its the mime type
                $mimeType = file_exists($file . '.' . $version->getExtension()) ? mime_content_type($file . '.' . $version->getExtension()) : null;

                $temp['width'] = (int) $version->getWidth();
                $temp['height'] = (int )$version->getHeight();
                $temp['version'] = (int) $version->getVersion();
                $temp['versionType'] = $version->getVersionType();
                $temp['extension'] = $version->getExtension();
                $temp['isDefault'] = $version->getIsDefault();
                $temp['size'] = $version->getSize();
                $temp['mimeType'] = $mimeType;
                $temp['bitrate'] = (int) $version->getBitrate();
                $temp['source'] = $source . '.' . $version->getExtension();
                return $temp;

            }, $video->versions([$v], 'version', SORT_ASC));

        }

        return $ver;
    }

    public function on()
    {
        $pairs = \Engine\Job\Publish\Support\Video::existIn($this->compose->getId());
        return array_map(function($pair){
            return [(int)$pair['id_channel'], (int)$pair['id_playlist'], (int)$pair['position_composed'], (int)$pair['position_playlist'], (int)$pair['position_channel']];
        }, $pairs);
    }

    public function composeVideo()
    {
        $videoArray = $this->video->toArray();
        $keys = ['id', 'idDirectory', 'size', 'bitrate', 'width', 'height', 'ratio', 'state', 'defaultThumbnail', 'codecAudio', 'codecVideo', 'audioBitrate', 'mimeType', 'registerDate','filename','extension'];
        $this->hideKeys($keys, $videoArray);

        return $videoArray;
    }

    public function metadata(/*Metadata*/ $metadata)
    {
        return [
            'image' => [
                'baseUrl' => $this->getAssetBaseUrl(),
                'file' => $metadata->getImage(),
                'alt' => $metadata->getImageAlt(),
                'title' => $metadata->getImageTitle(),
            ],
            'title' => $metadata->getTitle(),
            'description' => $metadata->getDescription(),
            'keywords' => $metadata->getKeywords()
        ];
    }

    public function body(/*L10n*/ $l10n)
    {

        $bodies['data'] = array_map(function(/*Body*/ $body){

            return $body->getBody();
//            return [
//                'id' => (int) $body->getId(),
//                'content' => $body->getBody()
//            ];

        }, $l10n->getBody());
        $bodies['count'] = sizeof($bodies['data']);

        return $bodies;
    }

    public function albums(/*L10n */$l10n)
    {
        $albums['data'] = array_map(function(/*Album*/ $album){

            $data = [
                //'id' => $album->getId(),
                'name' => $album->getName(),
                'description' => $album->getDescription(),
                'link' => $album->getLink(),
                'random' => $album->getRandom(),
                'cover' => [
                    'file' => $album->getCoverImage(),
                    'baseUrl' => $this->getAssetBaseUrl()
                ],
                'assets' => [
                    'count' => $album->getTotalImages()
                ]
            ];

            $query = new Query();
            $query->term = "";
            $query->by = Query::ALL;
            $query->from = 0;
            $query->to = -1;

            $basePath = $this->account->getConfig()->get(Config::STORAGE_ASSETS) . DIRECTORY_SEPARATOR . $this->account->getToken();
            $data['assets']['data'] = array_filter(array_map(function(Asset $asset) use ($basePath) {

                $filePath = $basePath . DIRECTORY_SEPARATOR . $asset->getFilename();

                // Check if the file exists
                if( file_exists($filePath) )
                {

                    $temp = [
                        //'id' => $asset->getId(),
                        'name' => $asset->getName(),
                        'description' => $asset->getDescription(),
                        'image' => [
                            'baseUrl' => $this->getAssetBaseUrl(),
                            'file' => $asset->getFilename(),
                            'alt' => $asset->getImageAlterText(),
                            'title' => $asset->getImageTitle(),
                            'date' => ['register' => (new DateTime($asset->getRegisterDate()))->format(parent::DATETIME_FORMAT)],
                            'size' => $asset->getSize(),
                            'width' => $asset->getWidth(),
                            'height' => $asset->getHeight(),
                            'extension' => $asset->getExtension(),
                            'mimeType' => $asset->getMimeType(),
                        ]
                    ];

                    return $temp;
                }
                else
                {
                    // Return empty in case of file not existence
                    return [];
                }

            }, $album->assets($query)), function($asset){
                return sizeof($asset);
            });

            return $data;

        }, $l10n->getAlbums());
        $albums['count'] = sizeof($albums['data']);
        return $albums;
    }

    public function assets(/*L10n*/ $l10n)
    {
        $assets = array_map(function(Asset $asset){

            return [
                //'id' => $asset->getId(),
                'name' => $asset->getName(),
                'description' => $asset->getDescription(),
                'image' => [
                    'baseUrl' => $this->getAssetBaseUrl(),
                    'file' => $asset->getFilename(),
                    'alt' => $asset->getImageAlterText(),
                    'title' => $asset->getImageTitle(),
                    'date' => ['register' => !$asset->getRegisterDate() ? null : $asset->getRegisterDate()->format(parent::DATETIME_FORMAT)]
                ]
            ];

        }, $l10n->getAssets());

        return ['count' => sizeof($assets) , 'data' => $assets];
    }

    public function playlists(/*Compose*/ $compose)
    {
        $playlists['data'] = array_map(function(/*Playlist*/ $playlist){

            $l10ns = $playlist->getL10n();
            $l10n = array_map(function($l10n) {

                $temp = $l10n->toArray();

                $temp2 = [];
                if( $temp['tags'] ) {
                    foreach ($temp['tags'] as $tag) {

                        $t = [];
                        $t['id'] = $tag['id'];
                        $t['name'] = $tag['name'];
                        $t['slug'] = $tag['nameStripped'];
                        $t['count'] = $tag['total'];

                        $temp2['tags'][] = $t;
                    }
                } else {
                    $temp2['tags'] = [];
                }

                unset($temp['tags']);
                $temp['tags'] = $temp2['tags'];

                $temp['date']['register'] = (new DateTime($temp['registerDate']))->format(parent::DATETIME_FORMAT);
                unset($temp['id']);
                unset($temp['registerDate']);

                $std = new \stdClass();
                $std->file = $temp['image'];
                $std->baseUrl = $this->getAssetBaseUrl();
                $temp['image'] = $std;
                return $temp;

            }, $l10ns);

            $data = [
                'id' => $playlist->getId(),
                'l10n' => $l10n,
                'slug' => str_slug(html_entity_decode($playlist->getManagementName(), ENT_COMPAT, "UTF-8"), "-"),
                // 'managementName' => $playlist->getManagementName(),
                'date' => [
                    'begin' => (new DateTime($playlist->getBeginDate()))->format(parent::DATETIME_FORMAT),
                    'end' => $playlist->getEndDate() == "" ? null : (new DateTime($playlist->getEndDate()))->format(parent::DATETIME_FORMAT)
                ],
                'channels' => array_map(function($channel){

                    /** @var Channel $channel */
                    $channel = Channel::find($channel->getId());

                    // Create a new ChannelSchema
                    $channelSchema = new ChannelSchema($this->account, $this->config, $channel, $channel->getRegisterDate(), null);

                    return $channelSchema->build()['data'];

                }, $playlist->channels())
            ];

            return $data;

        }, $compose->playlists());
        $playlists['count'] = sizeof($playlists['data']);
        return $playlists;
    }

    public function labels(/*Compose*/ $compose)
    {
        return  array_map(function($label){

            $l = $label->toArray();
            $l['slug'] = $l['nameStripped'];
            $l['count'] = $l['total'];

            unset($l['id']);
            unset($l['total']);
            unset($l['nameStripped']);
            unset($l['public']);
            unset($l['totalVideoCompose']);
            unset($l['totalLiveCompose']);

            return $l;
        }, $compose->labels());
    }

    /**
     * @param $compose
     * @return array
     * @throws \Exception
     */
    public function stickers(/*Compose*/ $compose)
    {

        $stickers = \Engine\Job\Publish\Support\Video::stickersIn($compose->getId());

        $temp = [];
        /** @var Sticker $sticker */
        foreach ($stickers as $sticker)
        {
            $t = $sticker->toArray();
            $t['total'] = $this->isLive ? $t['totalLive'] : $t['totalVideo'];
            $t['slug'] = $sticker->getNameStripped();
            unset($t['totalLive']);
            unset($t['totalVideo']);
            unset($t['nameStripped']);
            $temp[] = $t;
        }

        return $temp;
    }

    public function tags(/*L10n*/ $l10n)
    {
        $tags = array_map(function(/*Tag*/ $tag){

            return [
                //'id' => (int)$tag->getId(),
                'name' => $tag->getName(),
                'slug' => $tag->getNameStripped(),
                'count' => (int) $tag->getTotal()  // Count is the total number of times which the tag is used
            ];

        }, $l10n->getTags());
        // $tags['count'] = sizeof($tags['data']);
        return $tags;
    }

    private function hideKeys($keys, &$videoArray)
    {
        foreach ($keys as $key)
        {
            unset($videoArray[$key]);
        }
    }

}