<?php namespace Engine\Job\Publish;

use Doctrine\CouchDB\CouchDBClient;
use Exception;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Helper;
use Stage\Helper\Formating;
use Stage\Post\Dossier\Dossier;
use Stage\Post\Metadata\Metadata;
use Stage\Post\Post;

class V1 extends Publish implements PublishInterface {

    protected $account;

    protected $collections;

    public function __construct(Account $account)
    {
        parent::__construct($account, "");
    }

    public function addCollections($collections)
    {
        $this->collections = $collections;
    }

    public function getAccount()
    {
        return $this->account;
    }

    /**
     * @param $host
     * @param $port
     * @param $database
     * @return CouchDBClient
     * @throws Exception
     */
    public function couchClient($host, $port, $database)
    {

        try {

            $options = [
                'ip' => $host,
                'port' => $port,
                'dbname' => $database,
                'type' => "socket"  // stream|socket
            ];

            // Create a couch client
            return CouchDBClient::create($options);

        } catch(Exception $e) {
            throw $e;
        }

    }

    /**
     * @throws Exception
     */
    public function run()
    {

        $databases = ['stage_news_playlists', 'stage_news_channels', 'stage_news_latest', 'stage_news_channels_playlists'];
        try {

            \Engine\Job\Publish\Support\Post::publishPosts();

            $couchSettings = \Stage\Account\Settings\Couch::getInstance();
            $settings = $couchSettings->getSettings($this->account);

            foreach ($databases as $database)
            {
                // Create a couch client
                $client = $this->couchClient($settings->getHost(), $settings->getPort(), $database);
                if($database == 'stage_news_playlists') $this->newsPlaylists($client);
                if($database == 'stage_news_channels') $this->newsChannels($client);
                if($database == 'stage_news_latest') $this->newsLatest($client);
                if($database == 'stage_news_channels_playlists') $this->newsChannelPlaylists($client);
            }

        } catch(Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $client
     */
    public function newsPlaylists($client)
    {
        // Dossiers
        // ID_ACCOUNT-ID_CHANNEL-ID_PLAYLIST
        // For each collection
        $config = $this->getAccount()->getConfig();
        $cdn = $config->get(Config::ASSET_IMAGE);

        foreach ($this->collections as $collection)
        {

            $query = new \Stage\Post\Collection\Query();
            $query->term = "";
            $query->from = 0;
            $query->to = -1;

            // Get the collection dossiers
            $dossiers = $collection->dossiers($query);

            // For each dossier
            foreach ($dossiers as $dossier)
            {
                // Document name: IDACCOUNT-IDCOLLECTION-IDDOSSIER
                $document = $this->getAccount()->getId() . "-" . $collection->getId() . "-" . $dossier->getId();

                $query = new \Stage\Post\Dossier\Query();
                $query->term = "5";
                $query->by = \Stage\Post\Dossier\Query::STATE;
                $query->from = 0;
                $query->to = -1;

                // Get all the dossier posts
                $posts = $dossier->posts($query);

                // Convert the dossier to array
                $data = array_map(function ($post) use ($dossier, $cdn) {

                    return $this->postStructure($post, $dossier, $cdn);
//
                }, $posts);

                // $document = '100-1-35';  // ID_ACCOUNT - ID_CHANNEL(Collections) - ID_PLAYLIST(Dossiers)
                // 1. Need to find if the documents exists
                $this->saveToCouch($data, $client, $document);
            }
        }

    }

    /**
     * @param $client
     */
    public function newsChannels($client)
    {
        // Collections
        // For each collection
        foreach ($this->collections as $collection)
        {

            $query = new \Stage\Post\Collection\Query();
            $query->term = "";
            $query->from = 0;
            $query->to = -1;

            // Get the collection dossiers
            $dossiers = $collection->dossiers($query);

            // For each dossier
            foreach ($dossiers as $dossier)
            {

                // Document name: IDACCOUNT-IDCOLLECTION
                $document = $this->getAccount()->getId() . "-" . $collection->getId();

                // Get all iso languages
                $isoLanguages = Post::getAllIsoLanguages();

                $temp = [];
                $authors = [];
                $tagsList = [];
                $tags2 = [];
                foreach ($isoLanguages as $iso)
                {
                    $key = $iso['iso_language'];

                    $tagsList = \Engine\Job\Publish\Support\Post::postsTags($collection->getId(), $key);
                    foreach ($tagsList as $t)
                    {
                        $tags2[$key][] = ['id' => (int)$t['id_tag'],
                            'name' => $t['name'],
                            'nameStripped' => $t['name_stripped'],
                            'total' => (int)$t['total']];
                    }

                    $authorsList = \Engine\Job\Publish\Support\Post::postsAuthors($collection->getId(), $key);
                    foreach ($authorsList as $a)
                    {
                        $authors[$key][] = ['id' => (int)$a['id_author'],
                            'name' => $a['name'],
                            'email' => $a['email'],
                            'photo' => $a['photo'],
                            'website' => $a['website'],
                            'facebook' => $a['facebook'],
                            'twitter' => $a['twitter'],
                            'linkedin' => $a['linkedin'],
                            'instagram' => $a['instagram'],
                            'skype' => $a['skype'],
                            'role' => $a['role'],
                            'biography' => $a['biography'],
                            'total' => (int)$a['total']];
                    }
                }

                $data = [
                    'idChannel'=> $collection->getId(),
                    'name' => $collection->getName(),
                    'domain' => $collection->getDomain(),
                    'defaultIdPlaylist' => $collection->getDefaultDossier() instanceof Dossier ? $collection->getDefaultDossier()->getId() : null,
                    'tags' => $tags2,
                    'authors' => $authors
                ];

                // 1. Need to find if the documents exists
                $this->saveToCouch($data, $client, $document);

            }
        }
    }

    /**
     * @param $client
     */
    public function newsChannelPlaylists($client)
    {
        // CollectionsDossiers
        // For each collection
        foreach ($this->collections as $collection)
        {

            $query = new \Stage\Post\Collection\Query();
            $query->term = "";
            $query->from = 0;
            $query->to = -1;

            // Get the collection dossiers
            $dossiers = $collection->dossiers($query);

            // Document name: IDACCOUNT-IDCOLLECTION
            $document = $this->getAccount()->getId() . "-" . $collection->getId();

            $data = array_map(function($dossier){

                // Get all iso languages
                $isoLanguages = Post::getAllIsoLanguages();

                $temp = [];
                foreach ($isoLanguages as $iso)
                {
                    $key = $iso['iso_language'];
                    $tags = \Engine\Job\Publish\Support\Post::postsDossiersTags($dossier->getId(), $key);
                    foreach ($tags as $tag)
                    {
                        $temp[$key][$tag['name_stripped']] = ['count' => (int) $tag['total'], 'link' => $tag['name_stripped']];
                    }
                }

                $data = [
                    'idPlaylist' => $dossier->getId(),
                    'name' => $dossier->getName(),
                    'thumbnail' => "",
                    'random' => $dossier->getRandom(),
                    'tags' => $temp
                ];

                foreach ($isoLanguages as $iso)
                {
                    $link = str_replace(" ", "-", mb_strtolower(Formating::generateNameLink($dossier->getName()), 'UTF-8'));

                    $key = $iso['iso_language'];
                    $data[$key] = ['name' => $dossier->getName(),
                        'link' => $link];
                }

                return $data;

            }, $dossiers);


            // 1. Need to find if the documents exists
            $this->saveToCouch($data, $client, $document);

        }
    }

    /**
     * @param $client
     */
    public function newsLatest($client)
    {

        $config = $this->getAccount()->getConfig();
        $cdn = $config->get(Config::ASSET_IMAGE);

        // For each collection
        foreach ($this->collections as $collection)
        {

            $query = new \Stage\Post\Collection\Query();
            $query->term = "";
            $query->from = 0;
            $query->to = -1;

            // Get the collection dossiers
            $dossiers = $collection->dossiers($query);

            // For each dossier
            /*foreach ($dossiers as $dossier)
            {*/
            // Document name: IDACCOUNT-IDDOSSIER
            $document = $this->getAccount()->getId() . "-" . $collection->getId();

            // Get the latest posts
            $posts = \Engine\Job\Publish\Support\Post::postsLatest($collection->getId(), -1);

            // Make an unique array of posts
            $queue = [];
            $data = [];
            $temp = [];
            foreach ($posts as $post)
            {
                /*if( !in_array($post['id_post'], $queue) )
                {
                    array_push($queue, ['idPost' => $post['id_post'], 'idDossier' => $post['id_dossier']]);
                }*/

                // TODO Se já existir a linguage no Post entao nao faz a Post structure
                if( !in_array($post['id_post'], $temp) )
                {
                    // Find a Post
                    $p = Post::find($post['id_post']);
                    $dossier = Dossier::find($post['id_dossier']);

                    // Build the Post structure
                    $data[] = $this->postStructure($p, $dossier, $cdn);
                }

                $temp[] = $post['id_post'];
            }

            /*$data = [];
            foreach ($queue as $idPost)
            {
                // Find a Post
                $p = Post::find($idPost['idPost']);
                $dossier = Dossier::find($idPost['idDossier']);

                // Build the Post structure
                $data[] = $this->postStructure($p, $dossier, $cdn);
            }*/

            // 1. Need to find if the documents exists
            $this->saveToCouch($data, $client, $document);
            // }
        }
    }

    public function postStructure(Post $post, Dossier $dossier, $cdn)
    {
        $structure = [
            'idNew' => $post->getId(),
            'idPlaylist' => $dossier->getId()
        ];

        $postL10n = $post->getL10n();

        foreach ($postL10n as $l10n )
        {

            $metadata = $l10n->getMetadata() instanceof Metadata ? $l10n->getMetadata() : new Metadata();

            $keywords = str_replace(' ', '', $metadata->getKeywords());

            // Tags;
            $arrayTags = array_map(function ($tag){
                return $tag->getName();
            }, array_values(array_filter($l10n->getTags(), function ($tag) {
                return $tag->getPublic() == true;
            })));

            $tags = implode(",", $arrayTags);

            // Convert tags array to array stripped
            $tagsStripped = implode(",", array_map(function ($tag){
                return $tag->getNameStripped();
            }, array_values(array_filter($l10n->getTags(), function ($tag) {
                return $tag->getPublic() == true;
            }))));


            // Multimedia
            $multimedia = [];
            $multimedia[] = [
                'data'=> [
                    'src' => $l10n->getImage(),
                    'prefix' => $cdn . DIRECTORY_SEPARATOR . $this->account->getToken() . DIRECTORY_SEPARATOR

                ],
                'type' => 'image'
            ];

            $assets = $l10n->getAssets();
            foreach ($assets as $asset)
            {
                $multimedia[] = [
                    'data'=> [
                        'name' => $asset->getName(),
                        'description' => $asset->getDescription(),
                        'imageTitle' => $asset->getImageTitle(),
                        'imageAlterText' => $asset->getImageAlterText(),
                        'link' => $asset->getLink(),
                        'width' => $asset->getWidth(),
                        'height' => $asset->getHeight(),
                        'src' => $asset->getFilename(),
                        'prefix' => $cdn . DIRECTORY_SEPARATOR . $this->account->getToken() . DIRECTORY_SEPARATOR

                    ],
                    'text' => $asset->getName(),
                    'type' => 'image'
                ];
            }

            $readTime = 0;
            foreach($l10n->getBody() as $body)
            {
                $text = strip_tags($body->getBody());
                $multimedia[] = [
                    'text'=> $text,
                    'html' => $body->getBody(),
                    'type' => 'text'
                ];

                $readTime += Helper::readTime($text);

                $attributes['body'][] = $body->toArray();
            }

            $authors = $l10n->getAuthors();
            $auths = [];
            foreach ($authors as $author)
            {
                $roles = [];
                foreach ($author->getL10n() as $authorL10n)
                {
                    $roles[] = ['role'=> $authorL10n->getRole(), 'biography' => $authorL10n->getBiography(), 'iso' => $authorL10n->getIsoLanguage()];
                }

                $auths[] = [
                    'social'=> [
                        'facebook' => $author->getFacebook(),
                        'linkedin' => $author->getLinkedIn(),
                        'twitter' => $author->getTwitter(),
                        'instagram' => $author->getInstagram(),
                        'skype' => $author->getSkype()
                    ],
                    'id' => $author->getId(),
                    'name' => $author->getName(),
                    'email' => $author->getEmail(),
                    'photo' => $author->getPhoto(),
                    'roles' => $roles
                ];

            }

            //$readTime = $this->readTime($l10n->getBody());
            $link = str_replace(" ", "-", mb_strtolower(Formating::generateNameLink($l10n->getTitle()), 'UTF-8'));

            $meta = [
                'title' => $metadata->getTitle(),
                'description' => $metadata->getDescription(),
                'link' => $link,
                'data'=> [
                    'src' => $metadata->getImage(),
                    'prefix' => $cdn . DIRECTORY_SEPARATOR . $this->account->getToken() . DIRECTORY_SEPARATOR
                ]
            ];

            $structure[$l10n->getIsoLanguage()] = [
                'title' => $l10n->getTitle(),
                'description' => $l10n->getDescription(),
                'keywords' => $keywords,
                'tags' => $tags,
                'tagsStripped' => $tagsStripped,
                'publishDate' => date('Y-m-d', strtotime($l10n->getPublishDate())),
                'multimedia' => $multimedia,
                'readTime' => ['minutes' => $readTime],
                'metadata' => $meta,
                'authors' => $auths
            ];
        }

        return $structure;
    }

    public function saveToCouch($data, $client, $document)
    {

        $res = $client->findDocument($document);

        if( $res->status == 200 )
        {
            $rev = $res->body['_rev'];

            $result = $client->putDocument(array('_rev' => $rev, "data" => $data), $document);
        }
        else
        {
            // If the document not exists it will be created
            $result = $client->putDocument(array("data" => $data), $document);
            // TODO Pode dar erro 400 de dados mal formados
        }

        return $result;
    }
}